import { toJalaali, toGregorian, jalaaliMonthLength } from './jalali';
import { greg_to_islamic, islamic_to_greg, days_in_month } from './islamic';
export class Ymd {
}
export class Hms {
}
/**
 * Month number should be 1-based (unlike Date class)
 */
export class NiDatetime {
    constructor(date) {
        this.use(date);
    }
    use(date) {
        this.__date = date ? new Date(date) : date;
        return this;
    }
    set ymd(ymd) { }
    get ymd() { return this.__ymd(); }
    __ymd() { return { year: this.year, month: this.month, date: this.date }; }
    get hours12() { return this.hours > 12 ? this.hours % 12 : this.hours; }
    get hours() { return this.__date ? this.__date.getHours() : null; }
    set hours(hours) {
        this.__date = new Date(this.__date.getFullYear(), this.__date.getMonth(), this.__date.getDate(), hours, this.__date.getMinutes(), this.__date.getSeconds());
        this.use(this.__date);
    }
    get minutes() { return this.__date ? this.__date.getMinutes() : null; }
    set minutes(minutes) {
        this.__date = new Date(this.__date.getFullYear(), this.__date.getMonth(), this.__date.getDate(), this.__date.getHours(), minutes, this.__date.getSeconds());
        this.use(this.__date);
    }
    get seconds() { return this.__date ? this.__date.getSeconds() : null; }
    set seconds(seconds) {
        this.__date = new Date(this.__date.getFullYear(), this.__date.getMonth(), this.__date.getDate(), this.__date.getHours(), this.__date.getMinutes(), seconds);
        this.use(this.__date);
    }
    get am() { return this.hours < 12; }
    set am(toggle) { this.hours = this.hours12 + (toggle ? 0 : 12); }
    get pm() { return !this.am; }
    set pm(toggle) { this.hours = this.hours12 + (toggle ? 12 : 0); }
}
/**
 * Islamic Implemendation of NiDate
 */
export class NiIslamicDatetime extends NiDatetime {
    constructor(date) {
        super(date);
        this.__idate = { iy: null, im: null, id: null };
        this.use(date);
    }
    use(date) {
        this.__idate = date ? greg_to_islamic(date) : { iy: null, im: null, id: null };
        return super.use(date);
    }
    get year() { return this.__idate.iy; }
    get month() { return this.__idate.im; }
    get date() { return this.__idate.id; }
    get daysInMonth() {
        if (this.__idate.iy && this.__idate.im) {
            return days_in_month(this.__idate.iy, this.__idate.im);
        }
        else {
            return null;
        }
    }
    get weeksFirstday() {
        if (this.__idate.iy && this.__idate.im) {
            const greg = islamic_to_greg({ iy: this.__idate.iy, im: this.__idate.im, id: 1 });
            return new Date(greg.gy, greg.gm - 1, greg.gd).getDay();
        }
        else {
            return null;
        }
    }
    get weekDay() { return this.__date ? this.__date.getDay() : null; }
    get ymd() { return this.__ymd(); }
    set ymd(ymd) {
        const greg = islamic_to_greg({ iy: ymd.year, im: ymd.month, id: ymd.date });
        this.use(new Date(greg.gy, greg.gm - 1, greg.gd, 0, 0, 0));
    }
    clone() {
        const clone = new NiIslamicDatetime();
        if (this.__date) {
            clone.use(new Date(this.__date));
        }
        return clone;
    }
}
/**
 * Jalali Implmentation of NiDate
 */
export class NiJalaliDatetime extends NiDatetime {
    constructor(date) {
        super(date);
        this.__jdate = { jy: null, jm: null, jd: null };
        this.use(date);
    }
    use(date) {
        this.__jdate = date ? toJalaali(date) : { jy: null, jm: null, jd: null };
        return super.use(date);
    }
    get year() { return this.__jdate.jy; }
    get month() { return this.__jdate.jm; }
    get date() { return this.__jdate.jd; }
    get daysInMonth() {
        if (this.__jdate.jy && this.__jdate.jm) {
            return jalaaliMonthLength(this.__jdate.jy, this.__jdate.jm);
        }
        else {
            return null;
        }
    }
    get weeksFirstday() {
        if (this.__jdate.jy && this.__jdate.jm) {
            const greg = toGregorian(this.__jdate.jy, this.__jdate.jm, 1);
            return new Date(greg.gy, greg.gm - 1, greg.gd).getDay();
        }
        else {
            return null;
        }
    }
    get weekDay() { return this.__date ? this.__date.getDay() : null; }
    get ymd() { return this.__ymd(); }
    set ymd(ymd) {
        const greg = toGregorian(ymd.year, ymd.month, ymd.date);
        this.__date = new Date(greg.gy, greg.gm - 1, greg.gd, this.__date.getHours(), this.__date.getMinutes(), this.__date.getSeconds());
        this.use(this.__date);
    }
    clone() {
        const clone = new NiJalaliDatetime();
        if (this.__date) {
            clone.use(new Date(this.__date));
        }
        return clone;
    }
}
/**
 * Gregorian Implmentation of NiDate
 */
export class NiGregorianDatetime extends NiDatetime {
    constructor(date) {
        super(date);
    }
    get year() { return this.__date.getFullYear(); }
    get month() { return this.__date.getMonth() + 1; }
    get date() { return this.__date.getDate(); }
    get daysInMonth() { return new Date(this.year, this.month, 0).getDate(); }
    get weeksFirstday() { return new Date(this.year, this.__date.getMonth(), 1).getDay(); }
    get weekDay() { return this.__date.getDay(); }
    get ymd() { return this.__ymd(); }
    set ymd(ymd) {
        this.__date = new Date(ymd.year, ymd.month - 1, ymd.date, this.__date.getHours(), this.__date.getMinutes(), this.__date.getSeconds());
        this.use(this.__date);
    }
    clone() {
        const clone = new NiGregorianDatetime();
        if (this.__date) {
            clone.use(new Date(this.__date));
        }
        return clone;
    }
}
export const padNumber = (num, limit) => {
    return '0'.repeat(limit - num.toString().length) + num;
};
export const formatDate = (calendar, locale, format) => {
    // ---------- formats ----------
    const f = {
        mediumDate: () => `${f.WWW()} ${f.DD()} ${f.MMM()}, ${f.YYYY()}`,
        mediumTime: () => `${f.hh()}:${f.mm()} ${f.a()}`,
        shortDate: () => `${f.YY()}/${f.M()}/${f.D()}`,
        shortTime: () => `${f.h()}:${f.m()} ${f.a()}`,
        longDate: () => `${f.WWWW()} ${f.DD()} ${f.MMMM()}, ${f.YYYY()}`,
        longTime: () => `${f.hh()}:${f.mm()}:${f.ss()} ${f.A()}`,
        medium: () => `${f.mediumDate()}, ${f.mediumTime()}`,
        short: () => `${f.shortDate()} ${f.shortTime()}`,
        long: () => `${f.longDate()} ${f.longTime()}`,
        iso: () => `${f.YYYY()}-${f.MM()}-${f.DD()}${calendar.__date.toISOString().substring(10)}`,
        YYYY: () => padNumber(calendar.year, 4),
        MMMM: () => locale.monthsName[calendar.month - 1],
        YY: () => padNumber(calendar.year, 4).substring(2),
        MMM: () => locale.monthsNameShort[calendar.month - 1],
        MM: () => padNumber(calendar.month, 2),
        M: () => calendar.month,
        DD: () => padNumber(calendar.date, 2),
        D: () => calendar.date,
        WWWW: () => locale.daysName[calendar.weekDay],
        WWW: () => locale.daysNameShort[calendar.weekDay],
        WW: () => locale.daysNameMini[calendar.weekDay],
        HH: () => padNumber(calendar.hours, 2),
        hh: () => padNumber(calendar.hours12, 2),
        H: () => calendar.hours,
        h: () => calendar.hours12,
        mm: () => padNumber(calendar.minutes, 2),
        m: () => calendar.minutes,
        ss: () => padNumber(calendar.seconds, 2),
        s: () => calendar.seconds,
        A: () => locale.AMPM[calendar.hours > 12 ? 1 : 0],
        a: () => locale.ampm[calendar.hours > 12 ? 1 : 0],
        z: () => calendar.__date.toString().substring(25, 33),
    };
    // one format may generate a value that can be misinterpreted by another format
    // use placeholers to hide each format output until concatenated at the end
    const placeholders = {};
    let counter = 0;
    // --- IMPORTANT NOTE ---
    // f.keys should be checked in order
    // so the longer format strings be evaulated first
    // ----------------------
    // .sort((a, b) => b.length - a.length)
    Object.keys(f).forEach(key => {
        while (format.indexOf(key) >= 0) {
            const placeholder = `$[[${counter++}]]`; // generate a placeholder
            placeholders[placeholder] = f[key](); // format and memorize for placeholder
            format = format.replace(key, placeholder); // replace key with placeholder
        }
    });
    // replace placeholders with values
    Object.keys(placeholders).forEach(pholder => format = format.replace(pholder, placeholders[pholder]));
    return format;
};
const ɵ0 = () => new NiJalaliDatetime(), ɵ1 = () => new NiJalaliDatetime(), ɵ2 = () => new NiJalaliDatetime(), ɵ3 = () => new NiIslamicDatetime(), ɵ4 = () => new NiGregorianDatetime();
/**
 * Default NiDate Locales
 */
export const Locales = {
    fa_AF: {
        name: 'fa_AF',
        title: 'دری',
        new: ɵ0,
        week: '#',
        dir: 'rtl',
        firstday: 6,
        weekends: [5],
        daysName: ['یکشنبه', 'دوشنبه', 'سه شنبه', 'چهارشنبه', 'پنجشنبه', 'جمعه', 'شنبه'],
        daysNameShort: ['یک', 'دو', 'سه', 'چهار', 'پنج', 'جمعه', 'شنبه'],
        daysNameMini: ['ی', 'د', 'س', 'چ', 'پ', 'ج', 'ش'],
        monthsName: ['حمل', 'ثور', 'جوزا', 'سرطان', 'اسد', 'سنبله', 'میزان', 'عقرب', 'قوس', 'جدی', 'دلو', 'حوت'],
        monthsNameShort: ['حم', 'ثو', 'جو', 'سر', 'اس', 'سن', 'می', 'عق', 'قو', 'جد', 'دل', 'حو'],
        ampm: ['ق.ظ', 'ب.ظ'],
        AMPM: ['قبل از ظهر', 'بعد از ظهر'],
        today: 'امروز',
        clear: 'پاک'
    },
    ps_AF: {
        name: 'ps_AF',
        title: 'پښتو',
        new: ɵ1,
        week: '#',
        dir: 'rtl',
        firstday: 6,
        weekends: [5],
        daysName: ['یکښنبه', 'دوښنبه', 'سې ښنبه', 'چهارښنبه', 'پنجښنبه', 'جمعه', 'ښنبې'],
        daysNameShort: ['یک', 'دو', 'سې', 'چهار', 'پنج', 'جمعه', 'ښنبې'],
        daysNameMini: ['ی', 'د', 'س', 'چ', 'پ', 'ج', 'ښ'],
        monthsName: ['وری', 'غويی', 'غبرگولی', 'چنگاښ', 'زمری', 'وږی', 'تله', 'لړم', 'سلواغه', 'ليندۍ', 'مرغومی', 'کب'],
        monthsNameShort: ['وری', 'غويی', 'غبرگولی', 'چنگاښ', 'زمری', 'وږی', 'تله', 'لړم', 'سلواغه', 'ليندۍ', 'مرغومی', 'کب'],
        ampm: ['س', 'ښ'],
        AMPM: ['سهار', 'ماښام'],
        today: 'نن',
        clear: 'پاک'
    },
    fa_IR: {
        name: 'fa_IR',
        title: 'فارسی',
        new: ɵ2,
        week: '#',
        dir: 'rtl',
        firstday: 6,
        weekends: [5],
        daysName: ['یکشنبه', 'دوشنبه', 'سه شنبه', 'چهارشنبه', 'پنجشنبه', 'جمعه', 'شنبه'],
        daysNameShort: ['یک', 'دو', 'سه', 'چهار', 'پنج', 'جمعه', 'شنبه'],
        daysNameMini: ['ی', 'د', 'س', 'چ', 'پ', 'ج', 'ش'],
        monthsName: ['فروردین', 'اردیبهشت', 'خرداد', 'تیر', 'مرداد', 'شهریور',
            'مهر', 'آبان', 'آذر', 'دی', 'بهمن', 'اسفند'],
        monthsNameShort: ['فر', 'ار', 'خر', 'تی', 'مر', 'شه', 'مه', 'آب', 'آذ', 'دی', 'به', 'اس'],
        ampm: ['ق.ظ', 'ب.ظ'],
        AMPM: ['قبل از ظهر', 'بعد از ظهر'],
        today: 'امروز',
        clear: 'پاک'
    },
    ar_SA: {
        name: 'ar_SA',
        title: 'القمری',
        new: ɵ3,
        week: '#',
        dir: 'rtl',
        firstday: 0,
        weekends: [5, 6],
        daysName: ['الأحد', 'الإثنين', 'الثُلاثاء', 'الأربعاء', 'الخميس', 'الجُمْعَة', 'السبت'],
        daysNameShort: ['أحد', 'إثنين', 'ثُلاثاء', 'أربعاء', 'خميس', 'جُمْعَة', 'سبت'],
        daysNameMini: ['ح', 'ث', 'ثُ', 'ع', 'خ', 'جُ', 'س'],
        monthsName: ['محرم', 'صفر', 'ربيع الأول', 'ربيع الثاني', 'جمادى الأولى',
            'جمادى الآخرة', 'رجب', 'شعبان', 'رمضان', 'شوال', 'ذوالقعدة', 'ذوالحجة'],
        monthsNameShort: ['محرم', 'صفر', 'ربيع ١', 'ربيع ٢', 'جمادى ١',
            'جمادى ٢', 'رجب', 'شعبان', 'رمضان', 'شوال', 'ذو القعدة', 'ذو الحجة'],
        ampm: ['ق.ظ', 'ب.ظ'],
        AMPM: ['قبل الظهر', 'بعد الظهر'],
        today: 'اليوم',
        clear: 'نظيف'
    },
    en_US: {
        name: 'en_US',
        title: 'Gregorian',
        new: ɵ4,
        week: 'Wk',
        dir: 'ltr',
        firstday: 1,
        weekends: [6, 0],
        daysName: ['Sunday', 'Monday', 'Tuesday', 'Wedsday', 'Thursday', 'Friday', 'Saturday'],
        daysNameShort: ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'],
        daysNameMini: ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa'],
        monthsName: ['January', 'February', 'March', 'April', 'May', 'June',
            'July', 'August', 'September', 'October', 'November', 'December'],
        monthsNameShort: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'],
        ampm: ['am', 'pm'],
        AMPM: ['AM', 'PM'],
        today: 'Today',
        clear: 'Clear'
    }
};
export const toTimezone = (date, targetTimezoneUTCOffset) => {
    date = new Date(date.getTime());
    if (targetTimezoneUTCOffset) {
        // from local to utc
        date.setTime(date.getTime() + (date.getTimezoneOffset() * 60000));
        // from utc to target
        date.setTime(date.getTime() + (targetTimezoneUTCOffset * 60000));
    }
    return date;
};
export { ɵ0, ɵ1, ɵ2, ɵ3, ɵ4 };
//# sourceMappingURL=ni-datetime.js.map