export declare class Ymd {
    year: number;
    month: number;
    date: number;
}
export declare class Hms {
    hours: number;
    minutes: number;
    seconds: number;
    isPM: boolean;
}
/**
 * Month number should be 1-based (unlike Date class)
 */
export declare abstract class NiDatetime {
    __date: Date;
    constructor(date?: Date);
    use(date: Date): NiDatetime;
    abstract get year(): number;
    abstract get month(): number;
    abstract get date(): number;
    abstract get daysInMonth(): number;
    abstract get weeksFirstday(): number;
    abstract get weekDay(): number;
    set ymd(ymd: Ymd);
    get ymd(): Ymd;
    __ymd(): {
        year: number;
        month: number;
        date: number;
    };
    get hours12(): number;
    get hours(): number;
    set hours(hours: number);
    get minutes(): number;
    set minutes(minutes: number);
    get seconds(): number;
    set seconds(seconds: number);
    get am(): boolean;
    set am(toggle: boolean);
    get pm(): boolean;
    set pm(toggle: boolean);
    abstract clone(): NiDatetime;
}
export interface NiDatetimeLocale {
    name: string;
    new: () => NiDatetime;
    week: string;
    dir: string;
    firstday: number;
    weekends: number[];
    daysName: string[];
    daysNameShort: string[];
    daysNameMini: string[];
    monthsName: string[];
    monthsNameShort: string[];
    ampm: string[];
    AMPM: string[];
    today: string;
    clear: string;
}
/**
 * Islamic Implemendation of NiDate
 */
export declare class NiIslamicDatetime extends NiDatetime {
    __idate: {
        iy: any;
        im: any;
        id: any;
    };
    constructor(date?: Date);
    use(date: Date): NiDatetime;
    get year(): number;
    get month(): number;
    get date(): number;
    get daysInMonth(): number;
    get weeksFirstday(): number;
    get weekDay(): number;
    get ymd(): Ymd;
    set ymd(ymd: Ymd);
    clone(): NiDatetime;
}
/**
 * Jalali Implmentation of NiDate
 */
export declare class NiJalaliDatetime extends NiDatetime {
    __jdate: {
        jy: any;
        jm: any;
        jd: any;
    };
    constructor(date?: Date);
    use(date: Date): NiDatetime;
    get year(): number;
    get month(): number;
    get date(): number;
    get daysInMonth(): number;
    get weeksFirstday(): number;
    get weekDay(): number;
    get ymd(): Ymd;
    set ymd(ymd: Ymd);
    clone(): NiDatetime;
}
/**
 * Gregorian Implmentation of NiDate
 */
export declare class NiGregorianDatetime extends NiDatetime {
    constructor(date?: Date);
    get year(): number;
    get month(): number;
    get date(): number;
    get daysInMonth(): number;
    get weeksFirstday(): number;
    get weekDay(): number;
    get ymd(): Ymd;
    set ymd(ymd: Ymd);
    clone(): NiDatetime;
}
export declare const padNumber: (num: any, limit: number) => string;
export declare const formatDate: (calendar: NiDatetime, locale: NiDatetimeLocale, format: string) => string;
/**
 * Default NiDate Locales
 */
export declare const Locales: {
    fa_AF: {
        name: string;
        title: string;
        new: () => NiJalaliDatetime;
        week: string;
        dir: string;
        firstday: number;
        weekends: number[];
        daysName: string[];
        daysNameShort: string[];
        daysNameMini: string[];
        monthsName: string[];
        monthsNameShort: string[];
        ampm: string[];
        AMPM: string[];
        today: string;
        clear: string;
    };
    ps_AF: {
        name: string;
        title: string;
        new: () => NiJalaliDatetime;
        week: string;
        dir: string;
        firstday: number;
        weekends: number[];
        daysName: string[];
        daysNameShort: string[];
        daysNameMini: string[];
        monthsName: string[];
        monthsNameShort: string[];
        ampm: string[];
        AMPM: string[];
        today: string;
        clear: string;
    };
    fa_IR: {
        name: string;
        title: string;
        new: () => NiJalaliDatetime;
        week: string;
        dir: string;
        firstday: number;
        weekends: number[];
        daysName: string[];
        daysNameShort: string[];
        daysNameMini: string[];
        monthsName: string[];
        monthsNameShort: string[];
        ampm: string[];
        AMPM: string[];
        today: string;
        clear: string;
    };
    ar_SA: {
        name: string;
        title: string;
        new: () => NiIslamicDatetime;
        week: string;
        dir: string;
        firstday: number;
        weekends: number[];
        daysName: string[];
        daysNameShort: string[];
        daysNameMini: string[];
        monthsName: string[];
        monthsNameShort: string[];
        ampm: string[];
        AMPM: string[];
        today: string;
        clear: string;
    };
    en_US: {
        name: string;
        title: string;
        new: () => NiGregorianDatetime;
        week: string;
        dir: string;
        firstday: number;
        weekends: number[];
        daysName: string[];
        daysNameShort: string[];
        daysNameMini: string[];
        monthsName: string[];
        monthsNameShort: string[];
        ampm: string[];
        AMPM: string[];
        today: string;
        clear: string;
    };
};
export declare const toTimezone: (date: Date, targetTimezoneUTCOffset?: number) => Date;
